import { Http } from '@zeedhi/core';

class BudgetParameters {
  private budgetParameters = [];
  
  private budgetParametersHandled = [];
  
  public budgetMonths = [];
  
  public notRecharged = [];
  
  async getBudgetParameters() {
    const routeName = 'budgetParameters';
    const sendObject = {
      filter: [],
    };
    
    await Http.post(`/${routeName}`, sendObject).then((response) => {
      /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
      this.budgetParameters = response.data.dataset[routeName];
    });
  }

  get getParameters(): any {
    return this.budgetParameters[0];
  }
  
  public treatParameters(parameters: any) {
    const period = this.getParameters.IDPERIODOORCA;
    const filter = Object.entries(parameters).map(([key, values]) => {
      let aux;
      if (key === 'IDPERIODOORCA' && period) {
        aux = period;
      } else if (Array.isArray(values) || !values) {
        aux = key === 'IDPERIODOORCA' ? period : this.arrayToText(values, key);
      } else {
        aux = values;
      }
      return {
        name: key,
        operator: '=',
        value: aux,
      };
    });

    this.budgetParametersHandled = filter;
  }
  
  arrayToText(arr: any, key: string) {
    const aux = key === 'NRPERIODOORCA' ? 1 : null;
    return arr && arr.length > 0 ? `_${arr.map((obj) => obj[key]).join('_')}_` : aux;
  }

  get getParametersHandled(): any {
    return this.budgetParametersHandled;
  }
  
  public definesMonths(NRPERIODOORCA: number) {
    const months: string[] = [
      'JANEIRO', 'FEVEREIRO', 'MARCO', 'ABRIL', 'MAIO', 'JUNHO',
      'JULHO', 'AGOSTO', 'SETEMBRO', 'OUTUBRO', 'NOVEMBRO', 'DEZEMBRO',
    ];
    const period = this.getParameters.IDPERIODOORCA;
    const initialMonth = parseInt(this.getParameters.NRMESINIORCA, 10) - 1;
    const periodNumber = NRPERIODOORCA;
    
    let index = 0;
    let monthsCount = 0;

    switch (period) {
      case 'ANUAL':
        index = initialMonth;
        monthsCount = 12;
        break;
      case 'SEMESTRAL':
        index = (initialMonth + ((periodNumber - 1) * 6)) % 12;
        monthsCount = 6;
        break;
      case 'QUADRIMESTRAL':
        index = (initialMonth + ((periodNumber - 1) * 4)) % 12;
        monthsCount = 4;
        break;
      case 'TRIMESTRAL':
        index = (initialMonth + ((periodNumber - 1) * 3)) % 12;
        monthsCount = 3;
        break;
      case 'BIMESTRAL':
        index = (initialMonth + ((periodNumber - 1) * 2)) % 12;
        monthsCount = 2;
        break;
      case 'MENSAL':
        index = periodNumber - 1;
        monthsCount = 1;
        break;
      default:
        this.budgetMonths = [];
    }

    const result: string[] = [];
    for (let i = 0; i < monthsCount; i += 1) {
      result.push(months[(index + i) % 12]);
    }

    this.budgetMonths = result;
  }
  
  public getNotRecharged(text: string): boolean {
    return this.notRecharged.includes(text);
  }
  
  public setRecharged(text: string) {
    this.notRecharged = this.notRecharged.filter((item) => (item !== text));
  }
  
  public setNotRecharged() {
    this.notRecharged = [
      'gop10000_gridAdministrateTax',
    ];
  }
}

export default new BudgetParameters();
